from geoformat.conversion.bbox_conversion import (
    envelope_to_bbox,
    bbox_to_envelope,
    bbox_extent_to_2d_bbox_extent,
    bbox_to_polygon_coordinates
)

from test_all import test_function

envelope_to_bbox_parameters = {
    0: {
        "envelope": (-10, 10, -10, 10),
        "return_value": (-10, -10, 10, 10)
    },
    1: {
        "envelope": (0, 10, 0, 10),
        "return_value": (0, 0, 10, 10)
    },
    2: {
        "envelope": (-10, 0, -10,  0),
        "return_value": (-10, -10, 0, 0)
    },
    3: {
        "envelope": (-10, -10, 10, 10),
        "return_value": (-10, 10, -10, 10)
    },
    4: {
        "envelope": (0, 0, 10, 10),
        "return_value":  (0, 10, 0, 10)
    },
    5: {
        "envelope": (-10, -10, 0, 0),
        "return_value": (-10, 0, -10, 0)
    }
}

bbox_to_envelope_parameters = {
    0: {
        "bbox": (-10, 10, -10, 10),
        "return_value": (-10, -10, 10, 10)
    },
    1: {
        "bbox": (0, 10, 0, 10),
        "return_value": (0, 0, 10, 10)
    },
    2: {
        "bbox": (-10, 0, -10, 0),
        "return_value": (-10, -10, 0, 0)
    },
    3: {
        "bbox": (-10, -10, 10, 10),
        "return_value": (-10, 10, -10, 10)
    },
    4: {
        "bbox": (0, 0, 10, 10),
        "return_value":  (0, 10, 0, 10)
    },
    5: {
        "bbox": (-10, -10, 0, 0),
        "return_value": (-10, 0, -10, 0)
    }
}

bbox_extent_to_2d_bbox_extent_parameters = {
    0: {
        "bbox_extent": (-10, -10, -10, 10, 10, 10),
        "return_value":  (-10, -10, 10, 10)
    },
    1:  {
        "bbox_extent": (-10, -10, -10, 10, 10, 10, 10, 10),
        "return_value":  (-10, -10, 10, 10)
    },
}

bbox_to_polygon_coordinates_parameters = {
    0: {
        'bbox': (-10, -10, 10, 10),
        'return_value': [[[-10, -10], [-10, 10],  [10, 10], [10, -10], [-10, -10]]]
    },
}

def test_all():
    # bbox_to_envelope
    print(test_function(bbox_to_envelope, bbox_to_envelope_parameters))

    # envelope_to_bbox
    print(test_function(envelope_to_bbox, envelope_to_bbox_parameters))

    # bbox_extent_to_2d_bbox_extent
    print(test_function(bbox_extent_to_2d_bbox_extent, bbox_extent_to_2d_bbox_extent_parameters))

    # bbox_to_polygon_coordinates
    print(test_function(bbox_to_polygon_coordinates, bbox_to_polygon_coordinates_parameters))


if __name__ == '__main__':
    test_all()


from tests.data.geolayers import geolayer_attributes_only
from geoformat.conversion.fields_conversion import recast_field
from geoformat.driver.ogr.ogr_driver import geolayer_to_ogr_layer, ogr_layer_to_geolayer
from tests.geoformat.driver.ogr.compare_ogr_files import compare_geolayer, measure_time


def test_geojson_attributes(attributes_geolayer):

    # recast field_binary to string (because ogr does not allow binary field in geojson format)
    attributes_geolayer = recast_field(
        geolayer_to_recast=attributes_geolayer,
        field_name_to_recast='field_binary',
        recast_to_geoformat_type='String',
        resize_width=255
    )

    path_geolayer_attirbutes_only = 'data/geolayer_attributes_only.geojson'

    # write file
    geolayer_to_ogr_layer(
        geolayer=attributes_geolayer,
        path=path_geolayer_attirbutes_only,
        driver_name='geojson'
    )

    # open created just bellow geojson to geolayer (to compare to geolayer_attributes_only)
    ogr_geolayer_attributes = ogr_layer_to_geolayer(
        path=path_geolayer_attirbutes_only
    )

    # compare both geolayer
    compare_geolayer(attributes_geolayer, ogr_geolayer_attributes)


if __name__ == '__main__':
    test_geojson_attributes(attributes_geolayer=geolayer_attributes_only)




from geoformat.geoprocessing.measure.length import (
    segment_length,
)
from tests.data.segments import (
    segment_a,
    segment_b,
    segment_c,
    segment_d,
    segment_e,
    segment_f,
    segment_g,
    segment_h,
    segment_i
)
from test_all import test_function


segment_length_parameters = {
    0: {
        'segment': segment_a,
        'distance_type': 'EUCLIDEAN',
        'return_value': 1.
    },
    1: {
        'segment': segment_b,
        'distance_type': 'EUCLIDEAN',
        'return_value': 1.
    },
    2: {
        'segment': segment_c,
        'distance_type': 'EUCLIDEAN',
        'return_value': 1.
    },
    3: {
        'segment': segment_d,
        'distance_type': 'EUCLIDEAN',
        'return_value': 1.
    },
    4: {
        'segment': segment_e,
        'distance_type': 'EUCLIDEAN',
        'return_value': 2.8284271247461903
    },
    5: {
        'segment': segment_f,
        'distance_type': 'EUCLIDEAN',
        'return_value': 2.8284271247461903
    },
    6: {
        'segment': segment_g,
        'distance_type': 'EUCLIDEAN',
        'return_value': 2.8284271247461903
    },
    7: {
        'segment': segment_h,
        'distance_type': 'EUCLIDEAN',
        'return_value': 2.
    },
    8: {
        'segment': segment_i,
        'distance_type': 'EUCLIDEAN',
        'return_value': 2.
    }
}


def test_all():
    # segment length
    print(test_function(segment_length, segment_length_parameters))


if __name__ == '__main__':
    test_all()



import datetime

geoformat_field_type_to_python_type = {
    'Integer': int,
    'IntegerList': int,
    'Real': float,
    'RealList': float,
    'String': str,
    'StringList': str,
    'Binary': bytes,
    'Date': datetime.date,
    'Time': datetime.time,
    'DateTime': datetime.datetime,
    'Boolean': bool
}

geoformat_field_type_to_postgresql_type = {
    'Integer': 'integer',
    'IntegerList': "integer[]",
    'Real': 'numeric',
    'RealList': 'numeric[]',
    'String': 'character varying',
    'StringList': 'character varying[]',
    'Binary': 'bytea',
    'Date': 'date',
    'Time': 'time',
    'DateTime': 'timestamp',
    'Boolean': 'boolean'
}

geoformat_field_type_to_geojson_field_type = {
    'Binary': str,
    'Date': str,
    'Time': str,
    'DateTime': str,
}

python_type_to_field_type = {
    str: 'String',
    (str, list): 'StringList',
    float: 'Real',
    (float, list): 'RealList',
    int: 'Integer',
    (int, list): 'IntegerList',
    datetime.date: 'Date',
    datetime.time: 'Time',
    datetime.datetime: 'DateTime',
    bytes: 'Binary',
    bool: 'Boolean',
}

recast_black_list = {
    'Integer': {'Binary', 'Date', 'Time', 'DateTime'},
    'IntegerList': {'Real', 'Integer', 'Binary', 'Date', 'Time', 'DateTime', 'Boolean'},
    'Real': {'Binary', 'Date', 'Time', 'DateTime'},
    'RealList': {'Integer', 'Real', 'Binary', 'Date', 'Time', 'DateTime', 'Boolean'},
    'String': {'Date', 'Time', 'DateTime'},
    'StringList': {'Integer', 'Real', 'Binary', 'Date', 'Time', 'DateTime', 'Boolean'},
    'Date': {'Integer', 'IntegerList', 'Real', 'RealList', 'Binary', 'Time', 'DateTime'},
    'Time': {'Integer', 'IntegerList', 'Real', 'RealList', 'Binary', 'Date', 'DateTime'},
    'DateTime': {'Integer', 'IntegerList', 'Real', 'RealList', 'Binary'},
    'Binary': {'Integer', 'IntegerList', 'Real', 'RealList', 'Date', 'Time', 'DateTime'},
    'Boolean': {'IntegerList', 'RealList', 'StringList', 'Date', 'Time', 'DateTime'}
}
ogr_cod_field_type_to_geoformat_field_type = {
    0: 'Integer',
    1: 'IntegerList',
    2: 'Real',
    3: 'RealList',
    4: 'String',
    5: 'StringList',
    6: 'String',  # WideString
    7: 'StringList',  # WideStringList
    8: 'Binary',
    9: 'Date',
    10: 'Time',
    11: 'DateTime'
}
field_metadata_width_required = {'Real', 'RealList', 'String', 'StringList'}
field_metadata_precision_required = {'Real', 'RealList'}




from test_all import test_function

from geoformat.conf.format_data import (
    value_to_iterable_value,
    is_hexadecimal,
    pairwise
)

value_to_iterable_value_parameters = {
    0: {
        "value": 'foo',
        "return_value": ['foo']
    },
    1: {
        "value": 'foo',
        "output_iterable_type": list,
        "return_value": ['foo']
    },
    2: {
        "value": 'foo',
        "output_iterable_type": tuple,
        "return_value": ('foo',)
    },
    3: {
        "value": 'foo',
        "output_iterable_type": set,
        "return_value": {'foo'}
    },
    4: {
        "value": ['foo'],
        "return_value": ['foo']
    },
    5: {
        "value": ('foo',),
        "return_value": ['foo']
    },
    6: {
        "value": {'foo'},
        "return_value": ['foo']
    },
    7: {
        "value": ['foo'],
        "output_iterable_type": tuple,
        "return_value": ('foo',)
    },
    8: {
        "value": ('foo',),
        "output_iterable_type": tuple,
        "return_value": ('foo',)
    },
    9: {
        "value": {'foo'},
        "output_iterable_type": tuple,
        "return_value": ('foo',)
    },
    10: {
        "value": ['foo'],
        "output_iterable_type": set,
        "return_value": {'foo'}
    },
    11: {
        "value": ('foo',),
        "output_iterable_type": set,
        "return_value": {'foo'}
    },
    12: {
        "value": {'foo'},
        "output_iterable_type": set,
        "return_value": {'foo'}
    },
    13: {
        "value": ['foo', 'bar'],
        "output_iterable_type": set,
        "return_value": {'foo', 'bar'}
    },
    14: {
        "value": ('foo', 'bar'),
        "output_iterable_type": set,
        "return_value": {'foo', 'bar'}
    },
    15: {
        "value": {'foo', 'bar'},
        "output_iterable_type": set,
        "return_value": {'bar', 'foo'}
    },
    16: {
        "value": ['foo', 'bar'],
        "output_iterable_type": tuple,
        "return_value": ('foo', 'bar')
    },
    17: {
        "value": ('foo', 'bar'),
        "output_iterable_type": tuple,
        "return_value": ('foo', 'bar')
    },
    18: {
        "value": ['foo', 'bar'],
        "output_iterable_type": list,
        "return_value": ['foo', 'bar']
    },
    19: {
        "value": ('foo', 'bar'),
        "output_iterable_type": list,
        "return_value": ['foo', 'bar']
    },

}

is_hexadecimal_parameters = {
    0: {
        "s": "Hello world",
        "return_value": False
    },
    1: {
        "s": '1234567890',
        "return_value": True
    },
    2: {
        "s": "ABCDEFabcdef",
        "return_value": True
    },
    3: {
        "s": "ABCDEFabcdef1234567890",
        "return_value": True
    },
    4: {
        "s": "ABCDEFabcdef1234567890_",
        "return_value": False
    },
}

pairwise_parameters = {
    0: {
        "iterable": [0, 1, 2, 3, 4, 5, 6, 7, 8, 9],
        "return_value": [(0, 1), (1, 2), (2, 3), (3, 4), (4, 5), (5, 6), (6, 7), (7, 8), (8, 9)]
    },
    1: {
        "iterable": (0, 1, 2, 3, 4, 5, 6, 7, 8, 9),
        "return_value": [(0, 1), (1, 2), (2, 3), (3, 4), (4, 5), (5, 6), (6, 7), (7, 8), (8, 9)]
    }
}

def test_all():
    # value_to_iterable_value
    print(test_function(value_to_iterable_value, value_to_iterable_value_parameters))

    # is hexadecimal
    print(test_function(is_hexadecimal, is_hexadecimal_parameters))

    # pairwise
    print(test_function(pairwise, pairwise_parameters))


if __name__ == '__main__':
    test_all()


from geoformat.conf.format_data import value_to_iterable_value
from geoformat.conf.fields_variable import python_type_to_field_type


def geometries_scan_to_geometries_metadata(
    geometry_scan,
    crs=None,
    extent=True
):
    """
    Crete geometry metadata dict from geometry scan get from features_geometry_ref_scan.

    :param geometry_scan: result from features_geometry_ref_scan function
    :param crs: coordinates reference system of features contains in geolayer
    :param extent: if True add extent geolyer to geometry metadata
    :return: geometry metadata
    """
    if geometry_scan['type']:
        geometry_metadata = {'type': geometry_scan['type']}
        if extent and 'extent' in geometry_scan:
            if geometry_scan['extent']:
                geometry_metadata['extent'] = geometry_scan['extent']
        if crs:
            geometry_metadata['crs'] = crs
    else:
        geometry_metadata = None

    return geometry_metadata


def fields_scan_to_fields_metadata(fields_scan):
    """
    Deduce fields metadata from fields scan result.

    List of field's type and necessary keys :

        | type          | width    | precision | index    |
        |---------------|----------|-----------|----------|
        | 'Integer'     | None     | None      | Optional |
        | 'IntegerList' | None     | None      | Optional |
        | 'Real'        | Required | Required  | Optional |
        | 'RealList'    | Required | Required  | Optional |
        | 'String'      | Required | None      | Optional |
        | 'StringList'  | Required | None      | Optional |
        | 'Binary'      | None     | None      | Optional |
        | 'Date'        | None     | None      | Optional |
        | 'Time'        | None     | None      | Optional |
        | 'DateTime'    | None     | None      | Optional |
        | 'Boolean'     | None     | None      | Optional |

        example :
            field_metadata  = {
                'field_a': {'type': 'Integer', 'index': 0},
                'field_b': {'type': 'Real', 'width': 4, 'precision': 2, 'index': 1}
            }


    :param fields_scan: result of geolayer_fields_type_scan
    :return: geolayer fields metadata
    """
    # create fields metadata dict
    fields_metadata = {}

    # create field_name list ordered by 'field_index' in field_name data
    field_name_list = [None] * len(fields_scan)
    for i_field, (field_name, field_data) in enumerate(fields_scan.items()):
        # if "field_index" key in field_data we store field_name in field_name_list in 'field_index' order
        if 'field_index' in field_data:
            field_idx = field_data['field_index']
        else:
            field_idx = i_field
        field_name_list[field_idx] = field_name

    # loop on fields and create field metadata dict
    reorder_field_index = []
    for field_name in field_name_list:
        field_dict = fields_scan[field_name]
        if field_dict['field_delete'] is False:
            field_metadata_dict = {}
            # determine field type
            field_type = field_dict['field_type']
            field_type_key = field_type
            if field_dict['field_list'] is True:
                field_type_key = (field_type_key, list)
            # add data in field_metadata_dict
            # get type
            field_metadata_dict['type'] = python_type_to_field_type[field_type_key]
            # get width for str
            if field_type == str:
                field_metadata_dict['width'] = field_dict['field_width_str']
                # if we transform an list original to str (does not work with StringList type)
                if field_type is str and field_dict['field_list'] is False:
                    if field_dict['field_width_list'] and field_dict['field_width_list'] > field_dict['field_width_str']:
                        field_metadata_dict['width'] = field_dict['field_width_list']
            # get with and precision for float
            if field_type == float:
                field_metadata_dict['width'] = field_dict['field_width_float']
                field_metadata_dict['precision'] = field_dict['field_precision']
            # get field_index (if require)
            if 'field_index' in field_dict:
                field_metadata_dict['index'] = field_dict['field_index']

            # write field metadata in fields metadata
            fields_metadata[field_name] = field_metadata_dict

        else:
            # if field must be deleted and fields have an index (we have to reorder it)
            if 'field_index' in field_dict:
                reorder_field_index.append(field_dict['field_index'])

    # if we need to reorder field index
    if reorder_field_index:
        fields_metadata = reorder_metadata_field_index_after_field_drop(fields_metadata=fields_metadata,
                                                                        reorder_field_index=reorder_field_index)

    return fields_metadata


def reorder_metadata_field_index_after_field_drop(fields_metadata, reorder_field_index):
    """
    This function is used when a field is deleted in a geolayer.

    Reorder "index" key in fields_metadata for step(s) specified in reorder_field_index.

    :param fields_metadata: field metadata from geolayer
    :param reorder_field_index: list that contains original index to delete field.
    :return: fields_metadata with field index reordered after field delete
    """

    # init reorder field index
    reorder_field_index = value_to_iterable_value(reorder_field_index, list)

    # sort index
    reorder_field_index.sort()
    # loop on each index
    for idx in reversed(reorder_field_index):
        for field_name, field_metadata in fields_metadata.items():
            if field_metadata['index'] > idx:
                field_metadata['index'] = field_metadata['index'] - 1

    return fields_metadata


import geoformat
import time
import psycopg2


table_dict = {
    "ocean_0_r17_v18": None,
    "ocean_1_r16_v17": None,
    "ocean_2_r15_v16": None,
    "ocean_3_r14_v15": None,
    "ocean_5_r13_v14": None,
    "ocean_10_r12_v13": None,
    "ocean_20_r11_v12": None,
    "ocean_40_r10_v11": None,  ## start here
    "ocean_78_r9_v10": None,
    "ocean_156_r8_v9": None,
    "ocean_312_r7_v8": None,
    "ocean_625_r6_v7": None,
    "ocean_1250_r5_v6": None,
    "ocean_2500_r4_v5": None,
    "ocean_5000_r3_v4": None,
    "ocean_10000_r2_v3": None,
}





def get_geolayer_stats(geolayer):

    nb_features = len(geolayer['features'])
    nb_coordinates_list = [None] * nb_features
    area_list = [None] * nb_features
    geolayer_bbox = None
    min_coordinates = None
    min_coordinates_i_feat = None
    max_coordinates = None
    max_coordinates_i_feat = None
    for i, (i_feat, feature) in enumerate(geolayer['features'].items()):
        feature_geometry = feature.get('geometry')
        feature_nb_coordinates = 0.
        feature_area = 0.
        if feature_geometry:
            feature_nb_coordinates = len(list(geoformat.coordinates_to_point(feature_geometry['coordinates'])))
            feature_area = geoformat.geometry_area(feature_geometry)
            feature_bbox = feature_geometry['bbox']
            if geolayer_bbox is None:
                geolayer_bbox = feature_bbox
            else:
                geolayer_bbox = geoformat.bbox_union(geolayer_bbox,  feature_bbox)
            # compute min_coordinates
            if min_coordinates:
                if feature_nb_coordinates < min_coordinates:
                    min_coordinates = feature_nb_coordinates
                    min_coordinates_i_feat = i_feat
            else:
                min_coordinates = feature_nb_coordinates
                min_coordinates_i_feat = i_feat

            # compute max_coordinates
            if max_coordinates:
                if feature_nb_coordinates > max_coordinates:
                    max_coordinates = feature_nb_coordinates
                    max_coordinates_i_feat = i_feat
            else:
                max_coordinates = feature_nb_coordinates
                max_coordinates_i_feat = i_feat


        nb_coordinates_list[i] = feature_nb_coordinates
        area_list[i] = feature_area

    # coordinates stats
    sum_nb_coordinates = sum(nb_coordinates_list)
    mean_nb_coordinates = sum_nb_coordinates / nb_features
    # area stats
    sum_area = sum(area_list)
    mean_area = sum_area / nb_features
    return {
        'geolayer_name': geolayer['metadata']['name'],
        'geolayer_bbox': geolayer_bbox,
        'nb_features': float(nb_features),
        'nb_coordinates': float(sum_nb_coordinates),
        'min_coordinates': float(min_coordinates),
        'min_coordinates_i_feat': float(min_coordinates_i_feat),
        'mean_coordinates': float(mean_nb_coordinates),
        'max_coordinates': float(max_coordinates),
        'max_coordinates_i_feat': float(max_coordinates_i_feat),
        'sum_area': float(sum_area),
        'mean_area': float(mean_area),
    }


def launch_geoformat_simplify(geolayer, tolerance, algo='RDP'):

    geoalyer_nb_features =  len(geolayer['features'])
    simplify_time_list = [0] * len(geolayer['features'])
    simplify_time_max = None
    time_max_i_feat = None
    simplify_time_min = None
    time_min_i_feat = None
    for i, (i_feat, feature) in enumerate(geolayer['features'].items()):
        feature_geometry = feature.get('geometry')
        if feature_geometry:
            simplify_start = time.time()
            geoformat.simplify(geometry=feature_geometry, tolerance=tolerance, algo=algo)
            simplify_time = (time.time() - simplify_start) * 1000
            simplify_time_list[i] = simplify_time
            # compute time_max
            if simplify_time_max:
                if simplify_time > simplify_time_max:
                    simplify_time_max = simplify_time
                    time_max_i_feat = i_feat
            else:
                simplify_time_max = simplify_time
                time_max_i_feat = i_feat
            # compute time_min
            if simplify_time_min:
                if simplify_time < simplify_time_min:
                    simplify_time_min = simplify_time
                    time_min_i_feat = i_feat
            else:
                simplify_time_min = simplify_time
                time_min_i_feat = i_feat


    simplify_time_sum = sum(simplify_time_list)
    simplify_time_mean = simplify_time_sum/ geoalyer_nb_features
    simplify_time_list.sort()
    simplify_median = simplify_time_list[int(geoalyer_nb_features/2)]

    return {
        '{algo}_time_sum'.format(algo=algo): float(simplify_time_sum),
        '{algo}_time_min'.format(algo=algo): float(simplify_time_min),
        '{algo}_time_min_i_feat'.format(algo=algo): float(time_min_i_feat),
        '{algo}_time_mean'.format(algo=algo): float(simplify_time_mean),
        '{algo}_time_median'.format(algo=algo): float(simplify_median),
        '{algo}_time_max'.format(algo=algo): float(simplify_time_max),
        '{algo}_time_max_i_feat'.format(algo=algo): float(time_max_i_feat)
    }

def launch_postgresql_simplify(geolayer, cursor, tolerance, algo='st_simplify'):

    geoalyer_nb_features =  len(geolayer['features'])
    simplify_time_list = [0] * len(geolayer['features'])
    simplify_time_max = None
    time_max_i_feat = None
    simplify_time_min = None
    time_min_i_feat = None

    for i, (i_feat, feature) in enumerate(geolayer['features'].items()):
        feature_geometry = feature.get('geometry')
        if feature_geometry:
            geometry_wkb = geoformat.geometry_to_wkb(feature_geometry)
            simplify_start = time.time()
            request = "SELECT {algo}(%(geom)s::geometry, {tolerance});".format(
                algo=algo,
                tolerance=tolerance,
            )
            cursor.execute(
                request,
                {'geom': geometry_wkb.hex()}
            )
            simplify_time = (time.time() - simplify_start) * 1000
            simplify_time_list[i] = simplify_time
            # compute time_max
            if simplify_time_max:
                if simplify_time > simplify_time_max:
                    simplify_time_max = simplify_time
                    time_max_i_feat = i_feat
            else:
                simplify_time_max = simplify_time
                time_max_i_feat = i_feat
            # compute time_min
            if simplify_time_min:
                if simplify_time < simplify_time_min:
                    simplify_time_min = simplify_time
                    time_min_i_feat = i_feat
            else:
                simplify_time_min = simplify_time
                time_min_i_feat = i_feat


    simplify_time_sum = sum(simplify_time_list)
    simplify_time_mean = simplify_time_sum/ geoalyer_nb_features
    simplify_time_list.sort()
    simplify_median = simplify_time_list[int(geoalyer_nb_features/2)]

    return {
        '{algo}_time_sum'.format(algo=algo): float(simplify_time_sum),
        '{algo}_time_min'.format(algo=algo): float(simplify_time_min),
        '{algo}_time_min_i_feat'.format(algo=algo): float(time_min_i_feat),
        '{algo}_time_mean'.format(algo=algo): float(simplify_time_mean),
        '{algo}_time_median'.format(algo=algo): float(simplify_median),
        '{algo}_time_max'.format(algo=algo): float(simplify_time_max),
        '{algo}_time_max_i_feat'.format(algo=algo): float(time_max_i_feat)
    }


if __name__ == '__main__':
    host = "localhost"
    user = 'redpig'
    pwd = 'Flostib4'
    port = 5433
    bd_name = 'world_ic_night_2006'
    schema = 'map'

    # connect to database
    conn_string = "host={host} user={user} dbname={dbname} password={password} port={port}".format(
        host=host,
        user=user,
        password=pwd,
        dbname=bd_name,
        port=port)

    pg_adress = 'PG: {conn_string} schemas={schemas}'.format(conn_string=conn_string, schemas=schema)
    print(conn_string)
    print(pg_adress)
    database_connection = psycopg2.connect(conn_string)
    cursor = database_connection.cursor()

    extent = (-10018754.0+10, -5009377.0+10, 5009377.0-10, 15028131.0-10)


    begin_loop_time = time.time()
    output_features_list = []
    for table_name, value in table_dict.items():
        open_geolayer_time = time.time()
        geolayer = geoformat.ogr_layer_to_geolayer(
            path=pg_adress,
            layer_id_or_name=table_name,
            bbox_extent=True,
            bbox_filter=extent,
            feature_limit=None
        )
        print(table_name)
        tolerance_rdp = int(table_name.split('_')[1])
        tolerance_vw = tolerance_rdp ** 2
        print('\t get geolayer time', time.time() - open_geolayer_time)
        output_feature = {'attributes': {}}
        for key, value in get_geolayer_stats(geolayer).items():
            print('\t', key, value)
            output_feature['attributes'][key] = value
        for key, value in launch_geoformat_simplify(geolayer, tolerance=tolerance_rdp, algo='RDP').items():
            print('\t', key, value)
            output_feature['attributes'][key] = value
        for key, value in launch_geoformat_simplify(geolayer, tolerance=tolerance_vw, algo='VW').items():
            print('\t', key, value)
            output_feature['attributes'][key] = value
        for key, value in launch_postgresql_simplify(geolayer, cursor=cursor, tolerance=tolerance_rdp, algo='st_simplify').items():
            print('\t', key, value)
            output_feature['attributes'][key] = value
        for key, value in launch_postgresql_simplify(geolayer, cursor=cursor, tolerance=tolerance_rdp, algo='st_simplifyVW').items():
            print('\t', key, value)
            output_feature['attributes'][key] = value

        output_features_list.append(output_feature)
    print('loop time', time.time() - begin_loop_time)
output_geolayer = geoformat.feature_list_to_geolayer(feature_list=output_features_list, geolayer_name='simplify_stat')
print(geoformat.print_features_data_table(geolayer=output_geolayer))

geoformat.geolayer_to_ogr_layer(geolayer=output_geolayer, path='simplify_stat_RDP_VW_simplify_simplifyVW.csv', driver_name='CSV')


from geoformat.geoprocessing.union import union_by_split

from tests.data.geometries import (
    test_polygon_a,
    test_polygon_b,
    test_polygon_c
)

from test_all import (
    test_function,
    test_dependencies
)

union_by_split_parameters = {
    0: {
        "geometry_list": [test_polygon_a, test_polygon_b],
        "split_factor": 2,
        "bbox": False,
        "return_value": {'type': 'Polygon', 'coordinates': [[[-2.0, 2.0], [2.0, 2.0], [2.0, 1.0], [3.0, 1.0], [3.0, -3.0], [-1.0, -3.0], [-1.0, -2.0], [-2.0, -2.0], [-2.0, 2.0]]]}
    },
    1: {
        "geometry_list": [test_polygon_a, test_polygon_b],
        "split_factor": 4,
        "bbox": False,
        "return_value": {'type': 'Polygon', 'coordinates': [[[-2.0, 2.0], [2.0, 2.0], [2.0, 1.0], [3.0, 1.0], [3.0, -3.0], [-1.0, -3.0], [-1.0, -2.0], [-2.0, -2.0], [-2.0, 2.0]]]}
    },
    2: {
        "geometry_list": [test_polygon_a, test_polygon_c],
        "split_factor": 4,
        "bbox": False,
        "return_value":  {'type': 'MultiPolygon', 'coordinates': [[[[-2.0, 2.0], [2.0, 2.0], [2.0, -2.0], [-2.0, -2.0], [-2.0, 2.0]]], [[[2.0, 4.0], [4.0, 4.0], [4.0, 2.0], [2.0, 2.0], [2.0, 4.0]]]]}
    }
}


def test_all():

    if test_dependencies()['ogr']:
        # union_by_split
        print(test_function(union_by_split, union_by_split_parameters))


if __name__ == '__main__':
    test_all()



from test_all import test_function
from geoformat.driver.csv_driver import csv_to_geolayer
from pathlib import Path

from tests.data.geolayers import geolayer_paris_velib, geolayer_paris_velib_str

file_path_base = Path(__file__).parent.parent.parent.parent.joinpath

csv_to_geolayer_parameters = {
    0: {
        "path": file_path_base('data/csv/velib-disponibilite-en-temps-reel.csv'),
        "delimiter": ';',
        "header": True,
        "field_name_filter": None,
        "force_field_conversion": False,
        "serialize": False,
        "return_value": geolayer_paris_velib_str
    },
    1: {
        "path": file_path_base('data/csv/velib-disponibilite-en-temps-reel.csv'),
        "delimiter": ';',
        "header": True,
        "field_name_filter": None,
        "force_field_conversion": True,
        "serialize": False,
        "return_value": geolayer_paris_velib
    }
}

def test_all():


    # csv_to_geolayer
    print(test_function(csv_to_geolayer, csv_to_geolayer_parameters))

if __name__ == '__main__':
    test_all()


import copy

from geoformat.geoprocessing.connectors.operations import (
    coordinates_to_point
)

from geoformat.conf.error_messages import import_pyproj_error

try:
    import pyproj

    import_pyproj_success = True
except ImportError:
    import_pyproj_success = False


def format_coordinates(
        coordinates_list_tuple,
        format_to_type=list,
        precision=None,
        delete_duplicate_following_coordinates=False,
        in_crs=None,
        out_crs=None
):
    """
    This function allow to :
        - reproject coordinates to in_crs to another out_crs
        - round coordinates
        - delete duplicate following coordinates in
        - change type of coordinate's storage

    :param coordinates_list_tuple: tuple or list containing coordinates
    :param format_to_type: coordinate storage data type (list (by default) or tuple)
    :param precision: desired number of decimal for coordinates
    :param delete_duplicate_following_coordinates: option to delete similar coordinates that follow each other
    :param in_crs:
    :param out_crs:
    :return: formatted coordinates
    """
    if import_pyproj_success is True:
        # TODO add delete duplicated part
        if coordinates_list_tuple:

            output_coordinates_list_tuple = copy.deepcopy(coordinates_list_tuple)
            point_list = False
            if isinstance(output_coordinates_list_tuple[0], (float, int)):
                point_list = True
                output_coordinates_list_tuple = [output_coordinates_list_tuple]

            if isinstance(output_coordinates_list_tuple[0][0], (int, float)):
                # reproject if necessary
                if in_crs is not None and out_crs is not None:
                    x_coords, y_coords = zip(*output_coordinates_list_tuple)
                    transform_coords = pyproj.transform(in_crs, out_crs, x_coords, y_coords, always_xy=True)
                    output_coordinates_list_tuple = zip(*transform_coords)

                if precision is not None:
                    # change precision
                    output_coordinates_list_tuple = [[round(xyz, precision) for xyz in coordinates] for coordinates in
                                                     output_coordinates_list_tuple]

                # FORMAT coordinates storage type
                # reformat coordinates storage type (list or tuple)
                if format_to_type:
                    output_coordinates_list_tuple = format_to_type(
                        [format_to_type(coordinates) for coordinates in output_coordinates_list_tuple])
            else:

                output_coordinates_list_tuple = [None] * len(coordinates_list_tuple)
                # here we first format type of storage of coordinates and precision
                for i_coord, coordinates in enumerate(coordinates_list_tuple):
                    coord_tuple = format_coordinates(
                        coordinates_list_tuple=coordinates,
                        format_to_type=format_to_type,
                        precision=precision,
                        delete_duplicate_following_coordinates=delete_duplicate_following_coordinates,
                        in_crs=in_crs,
                        out_crs=out_crs
                    )
                    output_coordinates_list_tuple[i_coord] = coord_tuple

                if format_to_type:
                    output_coordinates_list_tuple = format_to_type(output_coordinates_list_tuple)

            # reformat coordinates list to point
            if point_list is True:
                output_coordinates_list_tuple = output_coordinates_list_tuple[0]

            # DELETE DUPLICATE FOLLOWING COORDINATES
            # in step before if we reformat precision we can create duplicate coordinates then if
            # delete_duplicate_following_coordinates is True we have to make a second scan of coordinates
            # if we are at level of part of geometry (list that contains coordinates in it)
            if delete_duplicate_following_coordinates is True and isinstance(output_coordinates_list_tuple[0],
                                                                             (list, tuple)) and isinstance(
                    output_coordinates_list_tuple[0][0], (int, float)):
                # scan list of coordinates or list of list of coordinates (don't scan coordinate list)
                # if coordinates
                retype_to_tuple = False
                # transform tuple to list (because we need to edit coordinates)
                if isinstance(output_coordinates_list_tuple, tuple):
                    retype_to_tuple = True
                    output_coordinates_list_tuple = list(output_coordinates_list_tuple)

                # try to find duplicate coordinates index
                duplicate_coordinates_idx_list = []
                for idx_inside_coord, inside_coordinates in enumerate(output_coordinates_list_tuple):
                    if idx_inside_coord == 0:
                        duplicate_coordinates_idx_list = []
                    else:
                        # if coordinates are same that previous index of coordinates is save in
                        # duplicate_coordinates_idx_list (we delete this coordinates after the loop)
                        if previous_coordinates == inside_coordinates:
                            duplicate_coordinates_idx_list.append(idx_inside_coord)
                    previous_coordinates = inside_coordinates

                # delete duplicate
                if duplicate_coordinates_idx_list:
                    for idx in reversed(duplicate_coordinates_idx_list):
                        del output_coordinates_list_tuple[idx]

                # retype to tuple if output_coordinates_list_tuple is originaly a tuple
                if retype_to_tuple is True:
                    output_coordinates_list_tuple = tuple(output_coordinates_list_tuple)
        else:
            if format_to_type == list:
                output_coordinates_list_tuple = []
            else:
                output_coordinates_list_tuple = ()

    else:
        raise Exception(import_pyproj_error)

    return output_coordinates_list_tuple


def coordinates_to_2d_coordinates(coordinates_list):
    """
    Convert a coordinates list with x dimension to 2d dimension list

    :param coordinates_list: list of coordinates with 2 or more dimension
    :return: coordinates list with only 2 dimensions.
    """

    if coordinates_list:
        if isinstance(coordinates_list[0], (int, float)):
            new_coordinates = [coordinates_list[0], coordinates_list[1]]
        elif isinstance(coordinates_list[0], (tuple, list)):
            new_coordinates = [None] * len(coordinates_list)
            for i_coord, under_coordinates in enumerate(coordinates_list):
                new_coordinates[i_coord] = coordinates_to_2d_coordinates(under_coordinates)
        else:
            raise Exception('error your geometry in input is not correct')
    else:
        new_coordinates = []

    return new_coordinates


def coordinates_to_centroid(coordinates_list_tuple, precision=None):
    """
    Return the centroid of given coordinates list or tuple

    :param coordinates_list_tuple: (list or tuple) coordinates list or tuple
    :param precision: (int) precision of coordinates (number of decimal places)
    :return: (tuple) centroid
    """

    for i_point, point in enumerate(coordinates_to_point(coordinates_list_tuple)):
        if i_point == 0:
            mean_point = list(point)
        else:
            mean_point[0] += point[0]
            mean_point[1] += point[1]

    nb_coordinates = i_point + 1
    centroid = [mean_point[0] / nb_coordinates, mean_point[1] / nb_coordinates]

    if precision:
        centroid = format_coordinates(coordinates_list_tuple=centroid, precision=precision)

    return centroid
